// TsDescrambler.cpp: TSfXNufR[_
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsDescrambler.h"
#include "TsTable.h"
#include "TsPacket.h"


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// EMML
#define EMM_VALID_PERIOD			7UL			// 7day

// B-CASJ[hďK[hC^[o
#define BCAS_REFRESH_INTERVAL		1000UL		// 1s

// PMT҂pPbgJEg
#define PMT_WAIT_PACKET_COUNT		(3250 *  200 / TS_PACKET_SIZE)	//  200ms @ 26Mbps
#define ECM_WAIT_PACKET_COUNT		(3250 * 2000 / TS_PACKET_SIZE)	// 2000ms @ 26Mbps

// NbvtCNg}N
#define CLIPED_INCREMENT(V)			{if((V) < 0xFFFFFFFFUL)(V)++;}


/////////////////////////////////////////////////////////////////////////////
// TSfXNufR[_
/////////////////////////////////////////////////////////////////////////////

const bool CTsDescrambler::InputMedia(IMediaData *pMediaData, const DWORD dwInputIndex)
{
	// ̓p[^`FbN
	ITsPacket * const pPacket = dynamic_cast<ITsPacket *>(pMediaData);
	if(!pPacket || (dwInputIndex >= GetInputNum()))return false;

	// PS
	InputPacket(pPacket);

	if(!m_bEnableBuffering || (m_DecBufState == BDS_RUNNING)){
		// pPbg
		OutputPacket(pPacket);
		}
	else{
		// pPbgobt@O
		PushUnprocPacket(pPacket);

		// obt@O
		BufManagement(pPacket);
		}

	return true;
}

const bool CTsDescrambler::OpenDescrambler(LPCTSTR lpszBCId)
{
	// UN[Y
	CloseDescrambler();

	IHalDevice *pHalDevice = NULL;

	try{
		// B-CASJ[h̃CX^X𐶐
		if(!(pHalDevice = ::BON_SAFE_CREATE<IHalDevice *>(lpszBCId)))throw __LINE__;

		// foCX^Cv`FbN
		if(pHalDevice->GetDeviceType() != ::BON_NAME_TO_GUID(TEXT("IHalBcasCard")))throw __LINE__;

		// foCX݂̑`FbN
		if(!pHalDevice->GetTotalDeviceNum())throw __LINE__;

		// foCX̋󂫂`FbN
		if(pHalDevice->GetActiveDeviceNum() >= pHalDevice->GetTotalDeviceNum())throw __LINE__;
	
		// IHalBcasCardC^tF[X擾
		if(!(m_pHalBcasCard = dynamic_cast<IHalBcasCard *>(pHalDevice)))throw __LINE__;

		// B-CASJ[hI[v
		if(!m_pHalBcasCard->OpenCard())throw __LINE__;
		
		// SԃZbg
		OnReset();
		}
	catch(...){
		// G[
		BON_SAFE_RELEASE(pHalDevice);
		m_pHalBcasCard = NULL;
		
		CloseDescrambler();
		return false;
		}

	return true;
}

const bool CTsDescrambler::CloseDescrambler(void)
{
	// B-CASJ[hN[Y
	BON_SAFE_RELEASE(m_pHalBcasCard);

	return true;
}

void CTsDescrambler::DiscardScramblePacket(const bool bEnable)
{
	// XNuRpPbgj邩ݒ
	m_bDiscardScramblePacket = (bEnable)? true : false;
}

void CTsDescrambler::EnableEmmProcess(const bool bEnable)
{
	// EMM邩ݒ
	m_bEnableEmmProcess = (bEnable)? true : false;
}

void CTsDescrambler::EnableBuffering(const bool bEnable)
{
	// obt@O邩ݒ
	m_bEnableBuffering = (bEnable)? true : false;
}

void CTsDescrambler::ResetStatistics(void)
{
	// vf[^Zbg
	m_dwInputPacketNum = 0UL;
	m_dwOutputPacketNum = 0UL;
	m_dwScramblePacketNum = 0UL;
	m_dwEcmProcessNum = 0UL;
	m_dwEmmProcessNum = 0UL;
	
	::ZeroMemory(m_adwInputPacketNum, sizeof(m_adwInputPacketNum));
	::ZeroMemory(m_adwOutputPacketNum, sizeof(m_adwOutputPacketNum));
	::ZeroMemory(m_adwScramblePacketNum, sizeof(m_adwScramblePacketNum));
}

const DWORD CTsDescrambler::GetDescramblingState(const WORD wProgramID)
{
	// PATe[u擾
	const IPatTable *pPatTable = dynamic_cast<const IPatTable *>(m_TsInputMap.GetMapTarget(0x0000U));
	if(!pPatTable)return IHalBcasCard::EC_BADARGUMENT;
		
	// ɈvPMTPID
	for(WORD wIndex = 0U ; wIndex < pPatTable->GetProgramNum() ; wIndex++){
		if(pPatTable->GetProgramID(wIndex) == wProgramID){
			
			// PMTe[u擾
			const IPmtTable *pPmtTable = dynamic_cast<const IPmtTable *>(m_TsInputMap.GetMapTarget(pPatTable->GetPmtPID(wIndex)));
			if(!pPmtTable)return IHalBcasCard::EC_BADARGUMENT;
			
			// ECMPID擾
			const IDescBlock * const pDescBlock = pPmtTable->GetPmtDesc();
			if(!pDescBlock)return IHalBcasCard::EC_BADARGUMENT;
	
			const ICaMethodDesc * const pCaMethodDesc = dynamic_cast<const ICaMethodDesc *>(pDescBlock->GetDescByTag(ICaMethodDesc::DESC_TAG));
			if(!pCaMethodDesc)return IHalBcasCard::EC_BADARGUMENT;

			const WORD wEcmPID = pCaMethodDesc->GetCaPID();
			if(wEcmPID >= TS_INVALID_PID)return IHalBcasCard::EC_NO_ERROR;

			// ECMvZbT擾
			const CEcmProcessor *pEcmProcessor = dynamic_cast<const CEcmProcessor *>(m_TsInputMap.GetMapTarget(pPatTable->GetPmtPID(wEcmPID)));
			if(!pEcmProcessor)return IHalBcasCard::EC_BADARGUMENT;
			
			// ԂԂ
			return pEcmProcessor->m_dwDescramblingState;
			}		
		}

	return IHalBcasCard::EC_BADARGUMENT;
}

const DWORD CTsDescrambler::GetInputPacketNum(const WORD wPID)
{
	// ̓pPbgԂ
	return (wPID < 0x2000U)? m_adwInputPacketNum[wPID] : m_dwInputPacketNum;
}

const DWORD CTsDescrambler::GetOutputPacketNum(const WORD wPID)
{
	// o̓pPbgԂ
	return (wPID < 0x2000U)? m_adwOutputPacketNum[wPID] : m_dwOutputPacketNum;
}

const DWORD CTsDescrambler::GetScramblePacketNum(const WORD wPID)
{
	// RpPbgԂ
	return (wPID < 0x2000U)? m_adwScramblePacketNum[wPID] : m_dwScramblePacketNum;
}

const DWORD CTsDescrambler::GetEcmProcessNum(void)
{
	// ECM񐔂Ԃ
	return m_dwEcmProcessNum;
}

const DWORD CTsDescrambler::GetEmmProcessNum(void)
{
	// EMM񐔂Ԃ
	return m_dwEmmProcessNum;
}

IHalBcasCard * CTsDescrambler::GetHalBcasCard(void)
{
	// B-CASJ[h[_foCXԂ
	return m_pHalBcasCard;
}

CTsDescrambler::CTsDescrambler(IBonObject *pOwner)
	: CMediaDecoder(pOwner, 1UL, 1UL)
	, m_TsInputMap(NULL)
	, m_TsOutputMap(NULL)
	, m_pHalBcasCard(NULL)
	, m_bDiscardScramblePacket(false)
	, m_bEnableEmmProcess(false)
	, m_wLastTsID(TS_INVALID_PID)
	, m_bEnableBuffering(false)
	, m_DecBufState(BDS_INITIAL)
	, m_dwPmtWaitCount(0UL)
{
	// vf[^
	ResetStatistics();

	// PATe[uPID}bvɒǉ
	m_TsInputMap.MapPid(0x0000U, dynamic_cast<ITsPidMapTarget *>(CPatTable::CreateInstance(dynamic_cast<IPatTable::IHandler *>(this))));

	// CATe[uPID}bvɒǉ
	m_TsInputMap.MapPid(0x0001U, dynamic_cast<ITsPidMapTarget *>(CCatTable::CreateInstance(dynamic_cast<ICatTable::IHandler *>(this))));

	// TOTe[uPID}bvɒǉ
	m_TsInputMap.MapPid(0x0014U, dynamic_cast<ITsPidMapTarget *>(CTotTable::CreateInstance(dynamic_cast<ITotTable::IHandler *>(this))));
}

CTsDescrambler::~CTsDescrambler(void)
{
	// B-CASJ[hN[Y
	CloseDescrambler();
	
	// obt@J
	ClearUnprocPacket();
}

const bool CTsDescrambler::OnPlay(void)
{
	// Ԃ
	return OnReset();
}

const bool CTsDescrambler::OnStop(void)
{
	// pPbgtbV
	FlushUnprocPacket();

	return CMediaDecoder::OnStop();
}

const bool CTsDescrambler::OnReset(void)
{
	// PID}bvZbg
	ResetPidMap();

	// TSIDZbg
	m_wLastTsID = TS_INVALID_PID;

	// PATZbg
	m_TsInputMap.ResetPid(0x0000U);

	// obt@OԃZbg
	m_DecBufState = BDS_INITIAL;
	m_dwPmtWaitCount = 0UL;
	ClearUnprocPacket();

	// vf[^Zbg
	ResetStatistics();

	return true;
}

void CTsDescrambler::OnPatTable(const IPatTable *pPatTable)
{
	// TSIDXVmF
	if(m_bEnableBuffering && (m_wLastTsID != pPatTable->GetTsID())){
		if(m_wLastTsID != TS_INVALID_PID){
			// pPbgtbV
			FlushUnprocPacket();
			m_DecBufState = BDS_INITIAL;
			m_dwPmtWaitCount = 0UL;
			
			// PID}bvZbg
			ResetPidMap();
			}
		m_wLastTsID = pPatTable->GetTsID();
		}

	// PMTe[uPID}bvǉ
	for(WORD wIndex = 0U ; wIndex < pPatTable->GetProgramNum() ; wIndex++){
		m_TsInputMap.MapPid(pPatTable->GetPmtPID(wIndex), dynamic_cast<ITsPidMapTarget *>(CPmtTable::CreateInstance(dynamic_cast<IPmtTable::IHandler *>(this))));
		}

	// obt@OԏԍXV
	if(m_DecBufState == BDS_INITIAL){
		m_DecBufState = BDS_STORING_PMT;
		m_dwPmtWaitCount = 0UL;
		}
}

void CTsDescrambler::OnCatTable(const ICatTable *pCatTable)
{
	// EMMPID擾
	const IDescBlock * const pDescBlock = pCatTable->GetCatDesc();
	if(!pDescBlock)return;
	
	const ICaMethodDesc * const pCaMethodDesc = dynamic_cast<const ICaMethodDesc *>(pDescBlock->GetDescByTag(ICaMethodDesc::DESC_TAG));
	if(!pCaMethodDesc)return;

	const WORD wEmmPID = pCaMethodDesc->GetCaPID();
	if(wEmmPID >= TS_INVALID_PID)return;

	// EMMvZbTPID}bvɒǉ
	m_TsInputMap.MapPid(wEmmPID, dynamic_cast<ITsPidMapTarget *>(CEmmProcessor::CreateInstance(dynamic_cast<ITsDescrambler *>(this))));
}

void CTsDescrambler::OnPmtTable(const IPmtTable *pPmtTable)
{
	// ECMPID擾
	const IDescBlock * const pDescBlock = pPmtTable->GetPmtDesc();
	if(!pDescBlock)return;
	
	const ICaMethodDesc * const pCaMethodDesc = dynamic_cast<const ICaMethodDesc *>(pDescBlock->GetDescByTag(ICaMethodDesc::DESC_TAG));
	if(!pCaMethodDesc)return;

	const WORD wEcmPID = pCaMethodDesc->GetCaPID();
	if(wEcmPID >= TS_INVALID_PID)return;

	// ECMvZbTmF
	if(!m_TsInputMap.GetMapTarget(wEcmPID)){
		// ECMvZbTPID}bvɒǉ
		m_TsInputMap.MapPid(wEcmPID, dynamic_cast<ITsPidMapTarget *>(CEcmProcessor::CreateInstance(dynamic_cast<ITsDescrambler *>(this))));
		}

	// ESvZbTPID}bvɒǉ
	for(WORD wIndex = 0U ; wIndex < pPmtTable->GetEsNum() ; wIndex++){
		m_TsOutputMap.MapPid(pPmtTable->GetEsPID(wIndex), dynamic_cast<ITsPidMapTarget *>(CEsProcessor::CreateInstance(m_TsInputMap.GetMapTarget(wEcmPID))));
		}
}

void CTsDescrambler::OnTotTable(const ITotTable *pTotTable)
{
	// Ȃ
}

void CTsDescrambler::InputPacket(ITsPacket * const pPacket)
{
	// PID}bvɓ
	if(m_pHalBcasCard)m_TsInputMap.StorePacket(pPacket);

	// ̓pPbgCNg
	CLIPED_INCREMENT(m_dwInputPacketNum);
	CLIPED_INCREMENT(m_adwInputPacketNum[pPacket->GetPID()]);
}

void CTsDescrambler::BufManagement(ITsPacket * const pPacket)
{
	if(m_PacketBuf.size() >= ECM_WAIT_PACKET_COUNT){
		// obt@O~bg
		FlushUnprocPacket();
		m_DecBufState = BDS_RUNNING;
		}
	else if(m_DecBufState == BDS_STORING_PMT){
		// PMT҂JE^CNg
		if(++m_dwPmtWaitCount >= PMT_WAIT_PACKET_COUNT){
			m_DecBufState = BDS_STORING_ECM;
			}
		}
	else if(m_DecBufState == BDS_STORING_ECM){
		// ECMXgAԃj^
		if(dynamic_cast<CEcmProcessor *>(m_TsInputMap.GetMapTarget(pPacket->GetPID()))){
			bool bNotStored = false;
		
			for(WORD wPID = 0x0000U ; wPID < 0x2000U ; wPID++){
				const CEcmProcessor *pEcmProcessor = dynamic_cast<CEcmProcessor *>(m_TsInputMap.GetMapTarget(wPID));
				if(pEcmProcessor){
					if(!pEcmProcessor->IsEcmReceived()){
						// ECMM
						bNotStored = true;
						}
					}
				}

			if(!bNotStored){
				m_DecBufState = BDS_RUNNING;
			
				// pPbgo
				FlushUnprocPacket();
				}
			}
		}
}

void CTsDescrambler::OutputPacket(ITsPacket * const pPacket)
{
	const WORD wPID = pPacket->GetPID();

	// ESvZbTɓ
	m_TsOutputMap.StorePacket(pPacket);

	if(pPacket->IsScrambled()){
		// RpPbgCNg
		CLIPED_INCREMENT(m_dwScramblePacketNum);
		CLIPED_INCREMENT(m_adwScramblePacketNum[wPID]);

		SendDecoderEvent(EID_CANT_DESCRAMBLE, reinterpret_cast<PVOID>(m_dwScramblePacketNum));
		
		// RpPbgjL͉ʃfR[_ɏo͂Ȃ
		if(m_bDiscardScramblePacket)return;
		}

	// o̓pPbgCNg
	CLIPED_INCREMENT(m_dwOutputPacketNum);
	CLIPED_INCREMENT(m_adwOutputPacketNum[wPID]);

	// ʃfR[_Ƀf[^o
	OutputMedia(dynamic_cast<IMediaData *>(pPacket));
}

void CTsDescrambler::ResetPidMap(void)
{
	// PAT/CAT/TOTȊOA}bv
	for(WORD wPID = 0x0002U ; wPID < 0x2000U ; wPID++){
		if(wPID != 0x0014U){
			m_TsInputMap.UnmapPid(wPID);
			}	
		}

	// CATZbg
	m_TsInputMap.ResetPid(0x0001U);

	// TOTZbg
	m_TsInputMap.ResetPid(0x0014U);
	
	// ESvZbTZbg
	m_TsOutputMap.ClearPid();
}

void CTsDescrambler::FlushUnprocPacket(void)
{
	// pPbgtbV
	for(DWORD dwIndex = 0UL ; dwIndex < m_PacketBuf.size() ; dwIndex++){
		OutputPacket(m_PacketBuf[dwIndex]);
		}

	// obt@NA
	ClearUnprocPacket();
}

void CTsDescrambler::PushUnprocPacket(const ITsPacket *pPacket)
{
	// pPbgobt@ɃvbV
	CTsPacket * const pNewPacket = new CTsPacket(NULL);
	::BON_ASSERT(pNewPacket);

	if(pNewPacket->CopyPacket(pPacket)){	
		m_PacketBuf.push_back(pNewPacket);
		}
}

void CTsDescrambler::ClearUnprocPacket(void)
{
	// pPbgNA
	for(DWORD dwIndex = 0UL ; dwIndex < m_PacketBuf.size() ; dwIndex++){
		BON_SAFE_RELEASE(m_PacketBuf[dwIndex]);
		}
	
	m_PacketBuf.clear();
}


/////////////////////////////////////////////////////////////////////////////
// ECMNX
/////////////////////////////////////////////////////////////////////////////

void CTsDescrambler::CEcmProcessor::OnPidReset(ITsPidMapper *pTsPidMapper, const WORD wPID)
{
	// L[NA(L[ɂjh~邽)
	m_Multi2Decoder.SetScrambleKey(NULL);
	m_dwDescramblingState = IHalBcasCard::EC_NO_ERROR;
	
	CPsiTableBase::OnPidReset(pTsPidMapper, wPID);
}

void CTsDescrambler::CEcmProcessor::Reset(void)
{
	m_bIsEcmReceived = false;
	
	CPsiTableBase::Reset();
}

CTsDescrambler::CEcmProcessor::CEcmProcessor(IBonObject *pOwner)
	: CPsiTableBase(NULL)
	, m_pTsDescrambler(dynamic_cast<CTsDescrambler *>(pOwner))
	, m_pHalBcasCard(m_pTsDescrambler->m_pHalBcasCard)
	, m_dwLastRetryTime(::GetTickCount() - BCAS_REFRESH_INTERVAL)
	, m_dwDescramblingState(IHalBcasCard::EC_NO_ERROR)
{
	// MULTI2fR[_
	m_Multi2Decoder.Initialize(m_pHalBcasCard->GetSystemKey(), m_pHalBcasCard->GetInitialCbc());
}

CTsDescrambler::CEcmProcessor::~CEcmProcessor(void)
{
	// ̃CX^XQƂĂCEsProcessorA}bv
	CEsProcessor *pEsProcessor;
	
	for(WORD wPID = 0x0000U ; wPID < 0x2000U ; wPID++){
		if(pEsProcessor = dynamic_cast<CEsProcessor *>(m_pTsDescrambler->m_TsOutputMap.GetMapTarget(wPID))){
			if(pEsProcessor->m_pEcmProcessor == this){
				m_pTsDescrambler->m_TsOutputMap.UnmapPid(wPID);
				}
			}
		}
}

const bool CTsDescrambler::CEcmProcessor::IsEcmReceived(void) const
{
	// ECM̎MLԂ
	return m_bIsEcmReceived;
}

const bool CTsDescrambler::CEcmProcessor::DescramblePacket(ITsPacket * const pTsPacket)
{
	// IMediaDataC^tF[X₢킹
	IMediaData *pMediaData = dynamic_cast<IMediaData *>(pTsPacket);

	// XNu
	if(pMediaData){
		if(m_Multi2Decoder.Decode(pTsPacket->GetPayloadData(), (DWORD)pTsPacket->GetPayloadSize(), pTsPacket->m_Header.byTransportScramblingCtrl)){
			// gX|[gXNuĐݒ
			pMediaData->SetAt(3UL, pMediaData->GetAt(3UL) & 0x3FU);
			pTsPacket->m_Header.byTransportScramblingCtrl = 0U;

			return true;
			}
		}
	
	return false;
}

const bool CTsDescrambler::CEcmProcessor::OnTableUpdate(const IPsiSection *pNewSection, const IPsiSection *pLastSection)
{
	// e[uID`FbN
	if(pNewSection->GetTableID() != TABLE_ID)return false;

	// ECMB-CASJ[hɓnăL[擾
	const BYTE *pKsData = m_pHalBcasCard->GetKsFromEcm(pNewSection->GetPayloadData(), pNewSection->GetPayloadSize());

	// ECMs͈xB-CASJ[hď
	if(!pKsData && (m_pHalBcasCard->GetEcmError() != IHalBcasCard::EC_NOT_CONTRACTED)){
		if((::GetTickCount() - m_dwLastRetryTime) >= BCAS_REFRESH_INTERVAL){
			// ďK[hC^[oo߂Ȃ			
			if(m_pHalBcasCard->OpenCard()){
				pKsData = m_pHalBcasCard->GetKsFromEcm(pNewSection->GetPayloadData(), pNewSection->GetPayloadSize());
				}
				
			// ŏIgCԍXV
			m_dwLastRetryTime = ::GetTickCount();
			}
		}

	// XNuL[XV
	m_Multi2Decoder.SetScrambleKey(pKsData);

	// Cxgʒm
	if(pKsData)CLIPED_INCREMENT(m_pTsDescrambler->m_dwEcmProcessNum);
	m_pTsDescrambler->SendDecoderEvent(EID_ECM_PROCESSED, reinterpret_cast<PVOID>(m_pHalBcasCard->GetEcmError()));

	// ԍXV
	m_dwDescramblingState = m_pHalBcasCard->GetEcmError();
	m_bIsEcmReceived = true;

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// EMMNX
/////////////////////////////////////////////////////////////////////////////

CTsDescrambler::CEmmProcessor::CEmmProcessor(IBonObject *pOwner)
	: CPsiTableBase(NULL)
	, m_pTsDescrambler(dynamic_cast<CTsDescrambler *>(pOwner))
	, m_pHalBcasCard(m_pTsDescrambler->m_pHalBcasCard)
{
	// Ȃ
}

CTsDescrambler::CEmmProcessor::~CEmmProcessor(void)
{
	// Ȃ
}

const bool CTsDescrambler::CEmmProcessor::OnTableUpdate(const IPsiSection *pNewSection, const IPsiSection *pLastSection)
{
	// e[uID`FbN
	if(pNewSection->GetTableID() != SECTION_TABLE_ID)return false;

	// EMM̂Ƃ͉Ȃ
	if(!m_pTsDescrambler->m_bEnableEmmProcess)return true;

	// ZNV
	const WORD wDataSize = pNewSection->GetPayloadSize();
	const BYTE *pHexData = pNewSection->GetPayloadData();
	
	WORD wPos = 0U;
	
	while((wDataSize - wPos) >= 17U){
		// EMMTCY`FbN
		const WORD wEmmSize = (WORD)pHexData[wPos + 6U] + 7U;
		if(((wDataSize - wPos) < wEmmSize) || (wEmmSize < 17U))break;

		// nhĂяo
		if(OnEmmBody(&pHexData[wPos], wEmmSize))break;
		
		// ͈ʒuXV
		wPos += wEmmSize;
		}

	return true;
}

const bool CTsDescrambler::CEmmProcessor::OnEmmBody(const BYTE *pEmmData, const WORD wEmmSize)
{
	// B-CASJ[hID擾
	const BYTE * const pCardID = m_pHalBcasCard->GetBcasCardID();

	// B-CASJ[hIDƏƍ
	for(DWORD dwPos = 0UL ; dwPos < 6UL ; dwPos++){
		if(pCardID[dwPos] != pEmmData[dwPos])return false;
		}

	// Xg[̎ԂmF
	CTsTime ValidTime;
	ValidTime.SetNowTime();									// ݎ擾
	ValidTime -= EMM_VALID_PERIOD * 24UL * 60UL * 60UL;		// LԂߋɖ߂
	
	const ITotTable *pTotTable = dynamic_cast<const ITotTable *>(m_pTsDescrambler->m_TsInputMap.GetMapTarget(0x0014U));
	if(!pTotTable)return true;
	CTsTime EmmTime = pTotTable->GetDateTime();				// TOT玞擾
	
	// LԂߋEMMf[^͏Ȃ
	if(EmmTime < ValidTime)return true;
	
	// B-CASJ[hEMMf[^M
	if(!m_pHalBcasCard->SendEmmSection(pEmmData, wEmmSize)){
		// gC
		if(!m_pHalBcasCard->SendEmmSection(pEmmData, wEmmSize)){
			// EMMs
			m_pTsDescrambler->SendDecoderEvent(EID_EMM_PROCESSED, reinterpret_cast<PVOID>(false));
			return true;
			}
		}

	// Cxgʒm
	CLIPED_INCREMENT(m_pTsDescrambler->m_dwEmmProcessNum);
	m_pTsDescrambler->SendDecoderEvent(EID_EMM_PROCESSED, reinterpret_cast<PVOID>(true));

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// ESNX
/////////////////////////////////////////////////////////////////////////////

CTsDescrambler::CEsProcessor::CEsProcessor(IBonObject *pOwner)
	: CBonObject(NULL)
	, m_pEcmProcessor(dynamic_cast<CEcmProcessor *>(pOwner))
{
	// Ȃ
}

CTsDescrambler::CEsProcessor::~CEsProcessor(void)
{
	// Ȃ
}

const bool CTsDescrambler::CEsProcessor::StorePacket(const ITsPacket *pTsPacket)
{
	// XNu
	return m_pEcmProcessor->DescramblePacket(const_cast<ITsPacket *>(pTsPacket));
}

void CTsDescrambler::CEsProcessor::OnPidReset(ITsPidMapper *pTsPidMapper, const WORD wPID)
{
	// Ȃ
}

void CTsDescrambler::CEsProcessor::OnPidMapped(ITsPidMapper *pTsPidMapper, const WORD wPID)
{
	// Ȃ
}

void CTsDescrambler::CEsProcessor::OnPidUnmapped(ITsPidMapper *pTsPidMapper, const WORD wPID)
{
	// CX^XJ
	Release();
}
